import 'package:flutter/widgets.dart';

/// [FormElement] is the most basic form element in Forma.
///
/// Every single element that can be added into a form has to extend [FormElement] in a way.
/// [FormElement] is responsible for initializing the widgets a certain element represents, create the state of the element (e.g. controllers) and dispose of the created state objects.
/// Via the [FormElement] the forms can retrieve values, build the form widgets and set values.
/// [T] represents the value the [FormElement] returns.
abstract class FormElement<T> {
  // Private variables
  Widget _widget;

  // Constructor
  FormElement();

  // Public methods

  /// Rebuild the widget the [FormElement] represents.
  ///
  /// After rebuilding the widget the widget getter of the [FormElement] is also updated to the newly created widget.
  Widget rebuild(BuildContext context) {
    _widget = build(context);
    return _widget;
  }

  // Abstract methods

  /// Initialize any state that the given [FormElement] requires.
  void initialize();

  /// Build function to build the [FormElement] into a widget. Dont call directly, call [rebuild] instead.
  Widget build(BuildContext context);

  /// Dispose the state of the [FormElement].
  void dispose();

  // Abstract getters
  T get value;
  set value(T value);

  // Getters
  /// The widget generated by the [FormElement].
  Widget get widget => _widget;
}
